package ch.unige.aou.controller.admin;

import java.text.ParseException;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.nimbusds.jwt.SignedJWT;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.Result;
import ch.unige.solidify.rest.Result.ActionStatus;
import ch.unige.solidify.security.AdminPermissions;
import ch.unige.solidify.security.RootPermissions;
import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.business.UserService;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.security.User;
import ch.unige.aou.model.security.UserDisplayableInfos;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.specification.UserSpecification;

@RestController
@ConditionalOnBean(AdminController.class)
@AdminPermissions
@SuppressWarnings("squid:S4684")
@RequestMapping(UrlPath.ADMIN_USERS)
public class UserController extends ResourceController<User> {
  static final Logger log = LoggerFactory.getLogger(UserController.class);

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @Override
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

  @UserPermissions
  @GetMapping(SolidifyConstants.URL_SEP + AouActionName.AUTHENTICATED)
  public HttpEntity<User> getAuthenticatedUser() {
    final String userId = this.getAuthenticatedUserId();
    final User user = ((UserService) this.itemService).findByExternalUid(userId);
    this.addLinks(user);
    return new ResponseEntity<>(user, HttpStatus.OK);
  }

  @UserPermissions
  @GetMapping(SolidifyConstants.URL_SEP + AouActionName.GET_USER_DISPLAYABLE_INFOS + SolidifyConstants.URL_SEP + "{externalUid}")
  public HttpEntity<UserDisplayableInfos> getUserDisplayableInfos(@PathVariable String externalUid) {
    final User user = ((UserService) this.itemService).findByExternalUid(externalUid);
    UserDisplayableInfos userInfos = new UserDisplayableInfos(user);
    return new ResponseEntity<>(userInfos, HttpStatus.OK);
  }

  @PostMapping(SolidifyConstants.URL_SEP + AouActionName.REVOKE_ALL_TOKENS + SolidifyConstants.URL_SEP + UrlPath.AUTH_USER_ID)
  public ResponseEntity<Object> revokeAllToken(@PathVariable String externalUid) {

    final User user = ((UserService) this.itemService).findByExternalUid(externalUid);
    if (user == null) {
      return ResponseEntity.notFound().build();
    }
    ((UserService) this.itemService).revokeAccessAndRefreshTokens(user);
    return ResponseEntity.ok().build();
  }

  @UserPermissions
  @PostMapping(SolidifyConstants.URL_SEP + AouActionName.REVOKE_MY_TOKENS)
  public ResponseEntity<Object> revokeMyToken(@RequestHeader("Authorization") String authorizationHeader) throws ParseException {
    final String token = authorizationHeader.substring(authorizationHeader.indexOf(' ') + 1);
    final String externalUid = SignedJWT.parse(token).getJWTClaimsSet().getClaim("user_name").toString();
    final User user = ((UserService) this.itemService).findByExternalUid(externalUid);
    ((UserService) this.itemService).revokeAccessAndRefreshTokens(user);
    return ResponseEntity.ok().build();
  }

  @PreAuthorize("@userPermissionService.isAllowedToUpdate(#id, #updateMap)")
  @Override
  public HttpEntity<User> update(@PathVariable String id, @RequestBody Map<String, Object> updateMap) {
    // get existing user
    final User existingUser = this.itemService.findOne(id);

    // Check if role changed
    String updateApplicationRoleId = ((UserService) this.itemService).getApplicationRoleId(updateMap);
    if (!existingUser.getApplicationRole().getResId().equals(updateApplicationRoleId)) {
      ((UserService) this.itemService).updateUserRole(existingUser.getExternalUid(), updateApplicationRoleId);
    }
    return super.update(id, updateMap);
  }

  @Override
  public HttpEntity<RestCollection<User>> list(@ModelAttribute User search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @GetMapping(params = "onlyWithValidationRights")
  public HttpEntity<RestCollection<User>> list(@RequestParam boolean onlyWithValidationRights, @ModelAttribute User search, Pageable pageable) {
    final UserSpecification spec = (UserSpecification) this.itemService.getSpecification(search);
    spec.setOnlyWithValidationRights(onlyWithValidationRights);
    final Page<User> listItem = this.itemService.findAll(spec, pageable);
    this.setResourceLinks(listItem);
    final RestCollection<User> collection = this.setCollectionLinks(listItem, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @Override
  public HttpEntity<User> get(@PathVariable String id) {
    return super.get(id);
  }

  @RootPermissions
  @PostMapping(AouActionName.REFRESH)
  public HttpEntity<Result> refresh() {
    Result result = new Result("users");
    try {
      Pageable pageable = PageRequest.of(0, 100);
      Page<User> userList;
      do {
        userList = this.itemService.findAll(pageable);
        for (User user : userList) {
          ((UserService) this.itemService).updateUser(user);
          this.itemService.save(user);
        }
        pageable = pageable.next();
      } while (!userList.isEmpty());
      result.setStatus(ActionStatus.EXECUTED);
      result.setMesssage("User information updated");
    } catch (RuntimeException e) {
      log.error("In updating user information", e);
      result.setStatus(ActionStatus.NOT_EXECUTED);
      result.setMesssage(e.getMessage());
    }
    return new ResponseEntity<>(result, HttpStatus.OK);
  }
}
