package ch.unige.aou.service.storage;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.OffsetDateTime;
import java.time.Period;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Profile;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.util.StreamUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyProcessingException;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.FileTool;
import ch.unige.solidify.util.StringTool;
import ch.unige.solidify.util.XMLTool;

import ch.unige.aou.AouConstants;
import ch.unige.aou.AouMetadataVersion;
import ch.unige.aou.AouXmlNamespacePrefixMapper;
import ch.unige.aou.business.PublicationService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.business.UserService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.security.User;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.model.xml.deposit.v2_4.DepositDoc;
import ch.unige.aou.model.xml.deposit.v2_4.FileType;
import ch.unige.aou.model.xml.fedora.repository.DatastreamProfile;
import ch.unige.aou.model.xml.fedora.repository.FedoraRepository;
import ch.unige.aou.service.MetadataService;

@Profile("storage-fedora3")
@Service
@ConditionalOnBean(AdminController.class)
public class Fedora3Storage extends StorageService {

  private static final Logger log = LoggerFactory.getLogger(Fedora3Storage.class);

  private static final String ADMIN_DATASTREAM_NAME = "ADMIN";
  private static final String ATTACHMENT_DATASTREAM_PREFIX = "ATTACHMENT";

  private String fedoraUrl;
  private String fedoraUser;
  private String fedoraPassword;
  private String fedoraPrefix;
  private JAXBContext jaxbContext;
  private final Map<String, String> xlstList = new HashMap<>();

  private StructureService structureService;

  public Fedora3Storage(AouProperties aouProperties, MessageService messageService, PublicationService publicationService,
          UserService userService, MetadataService metadataService, StructureService structureService) {
    super(messageService, publicationService, userService, metadataService);

    this.structureService = structureService;

    // Check configuration
    this.fedoraUrl = aouProperties.getStorage().getUrl();
    this.fedoraUser = aouProperties.getStorage().getUser();
    this.fedoraPassword = aouProperties.getStorage().getPassword();
    this.fedoraPrefix = aouProperties.getStorage().getPrefix();
    // Check Prefix
    if (StringTool.isNullOrEmpty(this.fedoraPrefix)) {
      throw new SolidifyRuntimeException("Missing Fedora prefix");
    }

    // Complete preloading configuration
    try {
      // Jaxb
      this.jaxbContext = JAXBContext.newInstance(DepositDoc.class);
      // Load XSL
      this.loadXsl();
    } catch (JAXBException | IOException e) {
      throw new SolidifyRuntimeException(e.getMessage(), e);
    }

    this.checkFedoraServer();
  }

  @Override
  public void generateExportMetadata(Publication publication) {
    Path metadataFolder = this.getMetadataFolder(publication.getResId());
    if (!FileTool.ensureFolderExists(metadataFolder)) {
      throw new SolidifyProcessingException("Problem with " + metadataFolder.toString());
    }
    // Generate Deposit metadata
    try {
      DepositDoc depositDoc = (DepositDoc) this.deserialize(publication.getMetadata());
      this.saveToFile(metadataFolder.resolve(AouConstants.AOU + SolidifyConstants.XML_EXT), this.serialize(depositDoc));
    } catch (JAXBException | IOException e) {
      throw new SolidifyProcessingException("Error in generation " + AouConstants.AOU + SolidifyConstants.XML_EXT + " metadata", e);
    }

    String completedPivotMetadata = this.getCompletedPivotMetadataForTransformation(publication);

    // Generate other metadata
    for (Entry<String, String> xslt : this.xlstList.entrySet()) {
      try {
        log.info("Generating metadata for {}", xslt.getKey());
        final String metadata = XMLTool.transform(completedPivotMetadata, xslt.getValue());
        this.saveToFile(metadataFolder.resolve(xslt.getKey() + SolidifyConstants.XML_EXT), metadata);
      } catch (IOException e) {
        throw new SolidifyProcessingException("Error in generation " + xslt.getKey() + SolidifyConstants.XML_EXT + " metadata", e);
      }
    }
  }

  @Override
  public void storePublication(Publication publication) throws IOException {
    try {
      Fedora3Client fedoraClt = this.getFedoraClient();
      this.setArchiveIdIfEmpty(fedoraClt, publication);
      this.createArchiveIfNotExist(fedoraClt, publication);
      this.saveMetadata(fedoraClt, publication);
      this.ingestAttachments(fedoraClt, publication);
    } catch (Exception e) {
      throw new SolidifyProcessingException(e.getMessage(), e);
    }
  }

  private void setArchiveIdIfEmpty(Fedora3Client fedora3Client, Publication publication) throws JAXBException {
    if (StringTool.isNullOrEmpty(publication.getArchiveId())) {
      // Get next PID (~identifier)
      String publicationPid = fedora3Client.getNextPID(this.fedoraPrefix);
      publication.setArchiveId(publicationPid);
      log.info("Fedora publication PID: {}", publicationPid);
    }
  }

  private void createArchiveIfNotExist(Fedora3Client fedora3Client, Publication publication) throws JAXBException {
    if (fedora3Client.check(publication.getArchiveId()) == HttpStatus.NOT_FOUND) {
      String result = fedora3Client.ingest(publication.getArchiveId(), publication.getTitle());
      if (!result.equals(publication.getArchiveId())) {
        throw new SolidifyProcessingException("Wrong PID: " + publication.getArchiveId() + " != " + result);
      }
      log.info("Fedora object {} created", publication.getArchiveId());
    }
  }

  private void ingestAttachments(Fedora3Client fedora3Client, Publication publication) {
    try {
      // get the attachments infos present in ADMIN datastream to ensure they correspond to the ATTACHMENTs to create
      Document adminDatastreamDocument = fedora3Client.getDataStreamContentAsXml(publication.getArchiveId(), ADMIN_DATASTREAM_NAME);
      XPath xpath = XMLTool.buildXPath();
      NodeList attachmentsList = (NodeList) xpath.compile("/admin/deposit/Attachment").evaluate(adminDatastreamDocument, XPathConstants.NODESET);

      int index = 0;
      index = this.ingestAttachmentsForDocumentFiles(fedora3Client, publication, attachmentsList, index);
      this.ingestAttachmentsForThumbnail(fedora3Client, publication, attachmentsList, index);

    } catch (XPathExpressionException e) {
      throw new SolidifyRuntimeException("Unable to get Attachment list from ADMIN datastream for object '" + publication.getArchiveId() + "'",
              e);
    }
  }

  private int ingestAttachmentsForDocumentFiles(Fedora3Client fedora3Client, Publication publication, NodeList attachmentsList, int index) {
    for (DocumentFile documentFile : publication.getDocumentFiles()) {

      Node attachmentNode = attachmentsList.item(index);
      String datastreamIdInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Datastream_ID");
      String fileNameInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./FileName");
      String sizeInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Size");
      String descriptionInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Description");

      index++;
      String attachmentId = String.format(ATTACHMENT_DATASTREAM_PREFIX + "%02d", index);
      String fileName = documentFile.getFileName();
      String size = documentFile.getFileSize().toString();
      String fileType = documentFile.getDocumentFileType().getValue();

      if (attachmentId.equals(datastreamIdInAdmin) && fileName.equals(fileNameInAdmin) && sizeInAdmin.equals(size) && descriptionInAdmin
              .equals(fileType)) {

        if (fedora3Client.checkDatastream(publication.getArchiveId(), attachmentId) == HttpStatus.NOT_FOUND) {
          DatastreamProfile dsProfile = fedora3Client
                  .addManagedDataStream(publication.getArchiveId(), attachmentId, fileName, documentFile.getFinalData());
          log.info("Fedora datastream {} ({}) added (size={}, mimeType={})", attachmentId, fileName, dsProfile.getDsSize(),
                  dsProfile.getDsMIME());
        } else {
          log.info("{} '{}' for object '{}' already exists", ATTACHMENT_DATASTREAM_PREFIX, attachmentId, publication.getArchiveId());
        }
      } else {
        String reason = "";
        if (!attachmentId.equals(datastreamIdInAdmin)) {
          reason += "(attachmentId: " + attachmentId + " != " + datastreamIdInAdmin + ")";
        }
        if (!fileName.equals(fileNameInAdmin)) {
          reason += "(filename: " + fileName + " != " + fileNameInAdmin + ")";
        }
        if (!sizeInAdmin.equals(size)) {
          reason += "(size: " + sizeInAdmin + " != " + size + ")";
        }
        if (!descriptionInAdmin.equals(fileType)) {
          reason += "(type: " + descriptionInAdmin + " != " + fileType + ")";
        }

        throw new SolidifyRuntimeException(
                "'" + datastreamIdInAdmin + "' described in ADMIN datastream does not correspond to the attachment '" + attachmentId
                        + "' to ingest " + reason);
      }
    }
    return index;
  }

  private void ingestAttachmentsForThumbnail(Fedora3Client fedora3Client, Publication publication, NodeList attachmentsList, int index) {
    if (publication.getThumbnail() != null) {

      Node attachmentNode = attachmentsList.item(index);
      String datastreamIdInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Datastream_ID");
      String fileNameInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./FileName");
      String sizeInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Size");
      String descriptionInAdmin = XMLTool.getFirstTextContent(attachmentNode, "./Description");

      index++;
      String attachmentId = String.format(ATTACHMENT_DATASTREAM_PREFIX + "%02d", index);
      String fileName = publication.getThumbnail().getFileName();
      String size = publication.getThumbnail().getFileSize().toString();
      String fileType = FileType.VIGNETTE.value();

      if (attachmentId.equals(datastreamIdInAdmin) && fileName.equals(fileNameInAdmin) && sizeInAdmin.equals(size) && descriptionInAdmin
              .equals(fileType)) {
        if (fedora3Client.checkDatastream(publication.getArchiveId(), attachmentId) == HttpStatus.NOT_FOUND) {

          ByteArrayResource content = new ByteArrayResource(publication.getThumbnail().getImage()) {
            @Override
            public String getFilename() {
              return fileName; // Filename has to be returned in order to be able to be posted
            }
          };

          DatastreamProfile dsProfile = fedora3Client.addManagedDataStream(publication.getArchiveId(), attachmentId, fileName, content);
          log.info("Fedora datastream {} ({}) for the publication thumbnail added (size={}, mimeType={})", attachmentId, fileName,
                  dsProfile.getDsSize(), dsProfile.getDsMIME());
        } else {
          log.info("{} '{}' for object '{}' already exists", ATTACHMENT_DATASTREAM_PREFIX, attachmentId, publication.getArchiveId());
        }
      } else {
        throw new SolidifyRuntimeException(
                "'" + datastreamIdInAdmin + "' described in ADMIN datastream does not correspond to the attachment '" + attachmentId
                        + "' to ingest '");
      }
    }
  }

  private void saveMetadata(Fedora3Client fedora3Client, Publication publication) {
    Path metadataFolder = this.getMetadataFolder(publication.getResId());
    for (Path xmlFile : FileTool.scanFolder(metadataFolder, Files::isRegularFile)) {
      if (xmlFile.getFileName().toString().endsWith(SolidifyConstants.XML_EXT)) {
        String dsId = xmlFile.getFileName().toString().substring(0, xmlFile.getFileName().toString().lastIndexOf(".")).toUpperCase();
        if (fedora3Client.checkDatastream(publication.getArchiveId(), dsId) == HttpStatus.NOT_FOUND) {
          DatastreamProfile dsProfile = fedora3Client
                  .addXmlDataStream(publication.getArchiveId(), dsId, publication.getTitle(), xmlFile.toUri());
          log.info("Fedora datastream {} ({}) added (size={}, mimeType={})", dsId, publication.getTitle(), dsProfile.getDsSize(),
                  dsProfile.getDsMIME());
        }
      }
    }
  }

  /************************************************************************************************/

  private void checkFedoraServer() {
    // Server info
    try {
      Fedora3Client fedoraClt = this.getFedoraClient();
      FedoraRepository repo = fedoraClt.getServerInfo();
      log.info("Fedora server version: {}", repo.getRepositoryVersion());
    } catch (Exception e) {
      throw new SolidifyRuntimeException("Error in connecting to Fedora server", e);
    }
  }

  private Fedora3Client getFedoraClient() {
    return new Fedora3Client(this.fedoraUrl, this.fedoraUser, this.fedoraPassword,
            this.metadataService.getAouXmlNamespacePrefixMapper(AouMetadataVersion.getDefaultVersion()));
  }

  private Path getMetadataFolder(String resId) {
    Path publicationPath = this.publicationService.getPublicationFilesFolderPath(resId);
    return Paths.get(publicationPath.toString(), AouConstants.METADATA_FOLDER);
  }

  private Marshaller getJaxbMarshaller() throws JAXBException {
    Marshaller marshaller = this.jaxbContext.createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
    return marshaller;
  }

  private Object deserialize(String xmlString) throws JAXBException {
    final StringReader newMd = new StringReader(xmlString);
    return this.jaxbContext.createUnmarshaller().unmarshal(newMd);
  }

  private String serialize(Object xmlObject) throws JAXBException {
    StringWriter sw = new StringWriter();
    Marshaller marshaller = this.getJaxbMarshaller();
    marshaller.marshal(xmlObject, sw);
    return sw.toString();
  }

  private void saveToFile(Path filePath, String fileContent) throws IOException {
    try (FileOutputStream file = new FileOutputStream(filePath.toFile())) {
      file.write(fileContent.getBytes());
    }
  }

  private void loadXsl() throws IOException {
    final ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver(this.getClass().getClassLoader());
    for (final org.springframework.core.io.Resource xsl : resolver
            .getResources("classpath*:/" + AouConstants.XSL_HOME + SolidifyConstants.URL_SEP + AouConstants.AOU.toLowerCase() + "2*.xsl")) {
      String fileName = xsl.getFilename();
      if (fileName == null) {
        throw new SolidifyRuntimeException("Resource " + xsl.getURI() + " has no filename");
      }
      log.info("Finding XML transformation: {}", fileName);
      this.xlstList.put(fileName.substring(fileName.indexOf("2") + 1, fileName.indexOf(".")),
              StreamUtils.copyToString(xsl.getInputStream(), Charset.defaultCharset()));
    }
  }

  /**
   * Generate XML completed with metadata required by the current public interface in production
   *
   * @param publication
   * @return
   */
  private String getCompletedPivotMetadataForTransformation(Publication publication) {
    try {
      Document document = XMLTool.parseXML(publication.getMetadata());
      this.completeDepositorInfos(publication, document);
      this.completePublicationDates(publication, document);
      this.completeEmbargoPeriods(publication, document);
      this.completeStructuresNamesWithAncestorsNames(publication, document);
      return XMLTool.documentToString(document);
    } catch (ParserConfigurationException | IOException | SAXException | XPathExpressionException | TransformerException e) {
      throw new SolidifyRuntimeException(e.getMessage());
    }
  }

  /**
   * Add depositor informations
   *
   * @param publication
   * @param doc
   * @throws XPathExpressionException
   */
  private void completeDepositorInfos(Publication publication, Document doc) throws XPathExpressionException {
    XPath xpath = this.getDepositDocXpath(publication);
    Node rootNode = (Node) xpath.compile("./aou_deposit:deposit_doc").evaluate(doc, XPathConstants.NODE);

    String personId = publication.getCreatorId();
    User user = this.userService.findByPersonResId(personId);

    String ns = this.metadataService.getAouXmlNamespacePrefixMapper(publication.getMetadataVersion()).getNamespacePrefixes()
            .get(AouConstants.XML_NAMESPACE_DEPOSIT_PREFIX);
    XMLTool.addChildNodeNS(rootNode, ns, "depositor_last_name", user.getLastName());
    XMLTool.addChildNodeNS(rootNode, ns, "depositor_first_name", user.getFirstName());
    XMLTool.addChildNodeNS(rootNode, ns, "depositor_unique_id", user.getUniqueId());
    XMLTool.addChildNodeNS(rootNode, ns, "depositor_email", user.getEmail());
  }

  /**
   * Add deposit creation and validation dates
   *
   * @param publication
   * @param doc
   * @throws XPathExpressionException
   */
  private void completePublicationDates(Publication publication, Document doc) throws XPathExpressionException {
    XPath xpath = this.getDepositDocXpath(publication);
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
    Node rootNode = (Node) xpath.compile("./aou_deposit:deposit_doc").evaluate(doc, XPathConstants.NODE);
    String ns = this.metadataService.getAouXmlNamespacePrefixMapper(publication.getMetadataVersion()).getNamespacePrefixes()
            .get(AouConstants.XML_NAMESPACE_DEPOSIT_PREFIX);
    XMLTool.addChildNodeNS(rootNode, ns, "creation_date", publication.getCreationTime().format(formatter));
    XMLTool.addChildNodeNS(rootNode, ns, "validation_date", OffsetDateTime.now().format(formatter));
  }

  /**
   * Calculate and add eventual embargoes durations in months from the embargo end date
   *
   * @param doc
   * @throws XPathExpressionException
   */
  private void completeEmbargoPeriods(Publication publication, Document doc) throws XPathExpressionException {
    XPath xpath = this.getDepositDocXpath(publication);
    NodeList fileNodes = (NodeList) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:files/aou_deposit:file")
            .evaluate(doc, XPathConstants.NODESET);
    for (int i = 0; i < fileNodes.getLength(); i++) {
      Node fileNode = fileNodes.item(i);
      Node embargoNode = (Node) xpath.compile("./aou_deposit:embargo").evaluate(fileNode, XPathConstants.NODE);
      if (embargoNode != null) {
        String endDateStr = (String) xpath.compile("./aou_deposit:end_date").evaluate(embargoNode, XPathConstants.STRING);
        int months = this.calculateMonthsDifferenceFromToday(endDateStr);
        String ns = this.metadataService.getAouXmlNamespacePrefixMapper(publication.getMetadataVersion()).getNamespacePrefixes()
                .get(AouConstants.XML_NAMESPACE_DEPOSIT_PREFIX);
        XMLTool.addChildNodeNS(embargoNode, ns, "duration_in_months", String.valueOf(months));
      }
    }
  }

  /**
   * For each metadata structure, prepend the structures path from top structure to metadata structure.
   * e.g: "Département de biologie moléculaire" --> "Faculté des sciences; Section de biologie; Département de biologie moléculaire"
   *
   * @param publication
   * @param doc
   * @throws XPathExpressionException
   */
  private void completeStructuresNamesWithAncestorsNames(Publication publication, Document doc) throws XPathExpressionException {
    XPath xpath = this.getDepositDocXpath(publication);
    NodeList academicStructuresNodes = (NodeList) xpath
            .compile("/aou_deposit:deposit_doc/aou_deposit:academic_structures/aou_deposit:academic_structure")
            .evaluate(doc, XPathConstants.NODESET);
    for (int i = 0; i < academicStructuresNodes.getLength(); i++) {
      Node structureNode = academicStructuresNodes.item(i);
      String structureId = (String) xpath.compile("./aou_deposit:resId").evaluate(structureNode, XPathConstants.STRING);
      List<Structure> ancestorStructures = this.structureService.getAncestorStructures(structureId);
      StringBuilder fullPathNameBuilder = new StringBuilder();
      ListIterator<Structure> li = ancestorStructures.listIterator(ancestorStructures.size());
      while (li.hasPrevious()) {
        fullPathNameBuilder.append(li.previous().getName() + "; ");
      }
      String structureName = (String) xpath.compile("./aou_deposit:name").evaluate(structureNode, XPathConstants.STRING);
      fullPathNameBuilder.append(structureName);
      Node structureNameNode = (Node) xpath.compile("./aou_deposit:name").evaluate(structureNode, XPathConstants.NODE);
      structureNameNode.setTextContent(fullPathNameBuilder.toString());
    }
  }

  private int calculateMonthsDifferenceFromToday(String endDateStr) {
    LocalDate endDate = LocalDate.parse(endDateStr).withDayOfMonth(1);

    // add one month to ensure the end date has passed
    endDate = endDate.plusMonths(1);

    LocalDate todayDate = LocalDate.now().withDayOfMonth(1);
    int months = Period.between(todayDate, endDate).getMonths();
    int years = Period.between(todayDate, endDate).getYears();
    return months + 12 * years;
  }

  private XPath getDepositDocXpath(Publication publication) {
    AouXmlNamespacePrefixMapper prefixMapper = this.metadataService.getAouXmlNamespacePrefixMapper(publication.getMetadataVersion());
    return XMLTool.buildXPath(prefixMapper);
  }
}
