package ch.unige.aou.service;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpMethod;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import ch.unige.solidify.rest.NoSqlResource;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.security.TokenUsage;
import ch.unige.solidify.util.AbstractRestClientTool;
import ch.unige.solidify.util.CollectionTool;

public abstract class NoSqlResourceClientService<T extends NoSqlResource> implements NoSqlResourceClientServiceInterface<T> {
  private static final int BUFFER_SIZE = 16384; // 2 ^ 14

  private static final Logger logger = LoggerFactory.getLogger(NoSqlResourceClientService.class);

  @Autowired
  protected AbstractRestClientTool restClientTool;

  @Autowired
  private Environment env;

  private String module;

  private Class<T> resourceClass;
  private final ObjectMapper mapper = new ObjectMapper();

  public NoSqlResourceClientService(Class<T> clazz, String module) {
    this.module = module;
    this.resourceClass = clazz;
    this.mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    this.mapper.registerModule(new JavaTimeModule());
  }

  public NoSqlResourceClientService(String module) {
    this.module = module;
  }

  protected <T> RestCollection<T> deserializeResponse(String jsonResult) {
    return (RestCollection<T>) this.deserializeResponse(jsonResult, this.resourceClass);
  }

  protected T getObjectFromUrl(String url, String jsonResult) {
    T result = null;
    try {
      result = this.mapper.readValue(jsonResult, this.resourceClass);
    } catch (IOException e) {
      logger.error("Error getting object from " + url, e);
    }
    return result;
  }

  protected <T> RestCollection<T> deserializeResponse(String jsonResult, Class<T> itemClass) {
    final List<T> list = CollectionTool.getList(jsonResult, itemClass);
    final RestCollectionPage page = CollectionTool.getPage(jsonResult);
    return new RestCollection<>(list, page);
  }

  protected void download(String urlString, Path path) {
    final String url = this.getBaseUrl() + urlString;
    this.restClientTool.downloadContent(URI.create(url), path, TokenUsage.WITH_TOKEN);
  }

  protected String getBaseUrl() {
    final String propertyName = "aou.module." + this.module + ".url";
    String url = this.env.getProperty(propertyName);

    if (url == null) {
      throw new IllegalStateException("Property aou.module." + this.module + ".url has to be defined.");
    }
    // Manage if there is a list of URLs
    if (url.contains(",")) {
      url = url.substring(0, url.indexOf(','));
    }

    return url;
  }

  protected String getModule() {
    return this.module;
  }

  protected String getResources(String url, Map<String, String> params) {
    final String finalUrl = url + "?size=" + RestCollectionPage.MAX_SIZE_PAGE;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.exchange(finalUrl, HttpMethod.GET, null, String.class, params).getBody();
  }

  protected String getResources(String url, Pageable pageable) {
    url = this.completeUrlWithPagination(url, null, pageable);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.exchange(url, HttpMethod.GET, null, String.class, new Object[0]).getBody();
  }

  protected <T> List<T> searchByProperties(String url, Map<String, String> params, Class<T> itemClass) {
    url += "?size=" + RestCollectionPage.MAX_SIZE_PAGE;
    for (final Map.Entry<String, String> entry : params.entrySet()) {
      url += "&" + entry.getKey() + "=" + entry.getValue();
    }
    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class);
    return CollectionTool.getList(jsonString, itemClass);
  }

  protected void setModule(String name) {
    this.module = name;
  }

  private String completeUrlWithPagination(String url, Map<String, String> properties, Pageable pageable) {
    final UriComponentsBuilder uriBuilder = UriComponentsBuilder.fromUriString(url);
    if (pageable != null) {
      final int page = pageable.getPageNumber();
      final int size = pageable.getPageSize();
      final Sort sort = pageable.getSort();
      final MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
      params.set("page", Integer.toString(page));
      params.set("size", Integer.toString(size));
      if (sort != null) {
        for (final Sort.Order order : sort) {
          final String sortProperty = order.getProperty();
          final String sortDirection = order.getDirection().toString();
          params.set("sort", sortProperty + "," + sortDirection);
        }
      }
      if (properties != null && !properties.isEmpty()) {
        params.setAll(properties);
      }
      uriBuilder.queryParams(params);
    }

    return uriBuilder.toUriString();
  }

}
