package ch.unige.aou.service;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Path;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.security.TokenUsage;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.rest.ModuleName;

public class ResourceBaseClientService<T extends Resource> {
  private static final Logger logger = LoggerFactory.getLogger(ResourceBaseClientService.class);

  protected final String resourcePath;

  protected AbstractRestClientTool restClientTool;

  private Environment env;

  private boolean isResourceServer = false;
  private final ObjectMapper mapper = new ObjectMapper();
  private final String module;
  private final Class<T> resourceClass;

  public ResourceBaseClientService(String resourcePath, Class<T> resourceClass, String module, AbstractRestClientTool restClientTool,
          Environment env) {
    this.module = module;
    this.resourcePath = resourcePath;
    this.resourceClass = resourceClass;
    this.restClientTool = restClientTool;
    this.env = env;
    this.mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    this.mapper.registerModule(new JavaTimeModule());
  }

  public void setResourceServer(boolean resourceServer) {
    this.isResourceServer = resourceServer;
  }

  protected String getBaseUrl() {
    return this.getBaseUrl(this.module);
  }

  protected String getBaseUrl(String module) {
    String propertyName = "aou.module." + module + ".url";
    String url = this.env.getProperty(propertyName);
    if (url == null) {
      throw new IllegalStateException("Property aou.module." + module + ".url has to be defined.");
    }
    // Manage if there is a list of URLs
    if (url.contains(",")) {
      url = url.substring(0, url.indexOf(','));
    }
    if (this.isResourceServer) {
      final int index = url.lastIndexOf('/');
      url = url.substring(0, index + 1).concat(ModuleName.RES_SRV);
    }
    return url;
  }

  protected Environment getEnv() {
    return this.env;
  }

  protected T getObjectFromUrl(String url, Map<String, String> params) {
    T result = null;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    try {
      final String jsonString = restTemplate.getForObject(url, String.class, params);
      if (jsonString == null) {
        return null;
      }
      result = this.mapper.readValue(jsonString, this.resourceClass);
    } catch (final IOException e) {
      logger.error("Error getting object from " + url, e);
    } catch (final HttpClientErrorException e) {
      if (e.getStatusCode() == HttpStatus.NOT_FOUND) {
        // No item found return null
        return null;
      } else {
        throw e;
      }
    }
    return result;
  }

  protected Class<T> getResourceClass() {
    return this.resourceClass;
  }

  protected T uploadFile(String url, org.springframework.core.io.Resource file, Class<T> itemClass) {
    final MultiValueMap<String, Object> parameters = new LinkedMultiValueMap<>();
    parameters.add("file", file);
    return this.uploadFile(url, file, itemClass, parameters);
  }

  protected T uploadFile(String url, org.springframework.core.io.Resource file, Class<T> itemClass, MultiValueMap<String, Object> parameters) {
    final HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.MULTIPART_FORM_DATA);

    parameters.add("file", file);

    final HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(parameters, headers);
    final RestTemplate restTemplate = this.restClientTool.getClient();

    final ResponseEntity<T> res = restTemplate.exchange(url, HttpMethod.POST, requestEntity, itemClass);
    return res.getBody();
  }

  protected void downloadDocumentFile(String url, Path path) {
    this.restClientTool.downloadContent(URI.create(url), path, TokenUsage.WITH_TOKEN);
  }

}
