package ch.unige.aou.model.settings;

import java.util.HashSet;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.util.SearchCriteria;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouSearchableResourceNormalized;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.specification.ResearchGroupSearchSpecification;

@Entity
public class ResearchGroup extends AouSearchableResourceNormalized<ResearchGroup> {

  @Size(min = 1)
  @NotNull
  @Column(name = AouConstants.DB_NAME)
  private String name;

  @Column(unique = true)
  private String code;

  @Column()
  private String acronym;

  @NotNull
  private Boolean active;

  /**
   * Simple users can create new research groups, but by default they are not validated
   */
  @NotNull
  private Boolean validated;

  @ManyToMany(mappedBy = "researchGroups")
  @JsonIgnore
  private Set<Person> people;

  @ManyToMany(mappedBy = "researchGroups")
  @JsonIgnore
  private Set<Publication> publications = new HashSet<>();

  /****************************************************************/

  @Override
  public void init() {

    if (this.active == null) {
      this.active = true;
    }

    if (this.validated == null) {
      this.validated = false;
    }
  }

  /****************************************************************/

  public Set<Person> getPeople() {
    return this.people;
  }

  public String getName() {
    return this.name;
  }

  public String getCode() {
    return this.code;
  }

  public String getAcronym() {
    return this.acronym;
  }

  public Boolean isActive() {
    return this.active;
  }

  public Boolean isValidated() {
    return this.validated;
  }

  public Set<Publication> getPublications() {
    return this.publications;
  }

  /****************************************************************/

  public void setName(String name) {
    this.name = name;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public void setAcronym(String acronym) {
    this.acronym = acronym;
  }

  public void setActive(Boolean active) {
    this.active = active;
  }

  public void setValidated(Boolean validated) {
    this.validated = validated;
  }

  public void setPublications(Set<Publication> publications) {
    this.publications = publications;
  }

  /****************************************************************/

  public boolean addPerson(Person p) {
    final boolean result = this.people.add(p);
    if (!p.getResearchGroups().contains(this)) {
      p.addResearchGroup(this);
    }
    return result;
  }

  public boolean removePerson(Person p) {
    final boolean result = this.people.remove(p);
    if (p.getResearchGroups().contains(this)) {
      p.removeResearchGroup(this);
    }
    return result;
  }

  public boolean addPublication(Publication p) {
    final boolean result = this.publications.add(p);
    if (!p.getResearchGroups().contains(this)) {
      p.addResearchGroup(this);
    }
    return result;
  }

  public boolean removePublication(Publication p) {
    final boolean result = this.publications.remove(p);
    if (p.getResearchGroups().contains(this)) {
      p.removeResearchGroup(this);
    }
    return result;
  }

  @Override
  public Specification<ResearchGroup> getSearchSpecification(SearchCriteria criteria) {
    return new ResearchGroupSearchSpecification(criteria);
  }

  @Override
  public void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly) {
    super.addLinks(linkBuilder, mainRes, subResOnly);
    if (mainRes) {
      this.add(linkBuilder.slash(this.getResId()).slash(ResourceName.PUBLICATIONS).withRel(ResourceName.PUBLICATIONS));
      this.add(linkBuilder.slash(this.getResId()).slash(ResourceName.PEOPLE).withRel(ResourceName.PEOPLE));
    }
  }
}
