package ch.unige.aou.model.tool;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.safety.Safelist;

import ch.unige.solidify.util.StringTool;

public class CleanTool {

  /**
   * List of different existing hyphens
   * <p>
   * –   en dash                   0x2013  &ndash;
   * —   em dash                   0x2014  &mdash;
   * ‒   figure dash               0x2012  &#8210;
   * ﹘  small em dash             0xFE58  &#65112;
   * -   hyphen-minus              0x2D    &#45;
   * ֊   armenian hyphen           0x58A   &#1418;
   * ‐   hyphen                    0x2010  &#8208;
   * ‑   non-breaking hyphen       0x2011  &#8209;
   * ⁃   hyphen bullet             0x2043  &#8259;
   * ﹣  small hyphen-minus        0xFE63  &#65123;
   * －  fullwidth hyphen-minus    0xFF0D  &#65293;
   */
  public static final List<String> hyphensList = List.of("–", "—", "‒", "﹘", "-", "֊", "‐", "‑", "⁃", "﹣", "－");

  private static Safelist titleJsoupSafeList = new Safelist().addTags("em", "i", "sub", "sup");

  private static Safelist abstractJsoupSafeList = new Safelist(titleJsoupSafeList).addTags("b", "strong", "p", "a", "li", "ul")
          .addAttributes("a", "href", "rel", "target")
          .addProtocols("a", "href", "http", "https");

  public static String cleanTitle(String value) {
    if (!StringTool.isNullOrEmpty(value)) {
      value = Jsoup.clean(value, CleanTool.titleJsoupSafeList);
      value = CleanTool.cleanHyphens(value);
      value = CleanTool.removeDoubleSpaces(value);
      value = CleanTool.removeFinalDot(value);
    }
    return value;
  }

  public static String cleanAbstract(String value) {
    if (!StringTool.isNullOrEmpty(value)) {
      value = Jsoup.clean(value, CleanTool.abstractJsoupSafeList);
      value = CleanTool.cleanHyphens(value);
    }
    return value;
  }

  public static String cleanString(String value) {
    if (!StringTool.isNullOrEmpty(value)) {
      value = CleanTool.trim(value);
      value = CleanTool.removeDoubleSpaces(value);
    }
    return value;
  }

  public static String cleanStringAndHyphens(String value) {
    value = CleanTool.cleanString(value);
    value = CleanTool.cleanHyphens(value);
    return value;
  }

  public static String cleanPaging(String value) {
    value = CleanTool.trim(value);
    value = CleanTool.removeDoubleSpaces(value);
    value = CleanTool.cleanHyphens(value);
    value = CleanTool.completeEndPageValue(value);
    return value;
  }

  /*******************************************************************************************/

  public static String trim(String value) {
    if (value != null) {
      value = value.trim();
    }
    return value;
  }

  public static String removeDoubleSpaces(String value) {
    if (value != null) {
      value = value.replaceAll("\\s+", " ");
    }
    return value;
  }

  public static String removeFinalDot(String value) {
    if (value != null && value.endsWith(".")) {
      value = value.substring(0, value.length() - 1);
    }
    return value;
  }

  /**
   * Replace hyphens of different kinds by a simple dash and remove surrounding spaces
   *
   * @param value
   * @return
   */
  public static String cleanHyphens(String value) {
    if (value != null) {
      for (String hyphen : CleanTool.hyphensList) {
        if (hyphen.equals("-")) {
          // escape simple hyphen to be used in regex
          hyphen = "\\\\" + hyphen;
        }
        value = value.replaceAll("\\s*[" + hyphen + "]\\s*", "-");
      }
    }
    return value;
  }

  /**
   * Complete end page after hyphen such as :
   * <p>
   * "p. 123-45" --> "p. 123-145"
   * but
   * "p. 123-17" isn't updated as the result would make no sense.
   *
   * @param value
   * @return
   */
  public static String completeEndPageValue(String value) {
    if (value != null) {
      Pattern p = Pattern.compile("\\d+\\-\\d+");
      Matcher m = p.matcher(value);
      if (m.find()) {
        String pages = m.group(0);
        String startPage = pages.substring(0, pages.indexOf("-"));
        StringBuilder endPageBuilder = new StringBuilder(pages.substring(pages.indexOf("-") + 1));
        while (startPage.length() > endPageBuilder.length()) {
          int unitToComplete = startPage.length() - endPageBuilder.length() - 1;
          endPageBuilder.insert(0, startPage.charAt(unitToComplete));
        }
        if (Integer.valueOf(startPage) <= Integer.valueOf(endPageBuilder.toString())) {
          String newPages = startPage + "-" + endPageBuilder;
          value = value.replace(pages, newPages);
        }
      }
    }
    return value;
  }

  public static String capitalizeString(String name) {
    if (name.equals(name.toUpperCase())) {
      StringBuilder str = new StringBuilder(name.substring(0, 1).toUpperCase() + name.substring(1).toLowerCase());
      Matcher matcher = Pattern.compile("[- \\']").matcher(name);
      // Check all occurrences
      while (matcher.find()) {
        str.setCharAt(matcher.start() + 1, Character.toUpperCase(name.charAt(matcher.start() + 1)));
      }
      return str.toString();
    }
    return name;
  }

  public static String removeTrailingDots(String value) {
    if (value != null) {
      value = value.replaceAll("\\.+$", "");
    }
    return value;
  }

  public static String removeTrailingParenthesisAndContent(String value) {
    if (value != null) {
      value = value.replaceAll("\\(.*\\)+$", "");
    }
    return value;
  }

  public static String removeInvisibleCharacters(String value) {
    if (value != null) {
      value = value.replaceAll("\\p{C}", "");
    }
    return value;
  }

  public static String removeTrailingColon(String value) {
    if (value != null) {
      value = value.replaceAll(":+$", "");
    }
    return value;
  }

}
