package ch.unige.aou.specification;

import java.time.OffsetDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.specification.SolidifySpecification;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.publication.Publication;

public class PublicationSpecification extends SolidifySpecification<Publication> {
  private static final long serialVersionUID = -2649320901418924922L;

  private String fullName;

  private OffsetDateTime lastUpdatedBefore;

  public PublicationSpecification(Publication criteria) {
    super(criteria);
  }

  @Override
  protected void completePredicatesList(Root<Publication> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    if (this.criteria.getCreator() != null) {
      predicatesList.add(builder.equal(root.get("creator").get("resId"), this.criteria.getCreator().getResId()));
    }
    if (this.criteria.getStatus() != null) {
      predicatesList.add(builder.equal(root.get("status"), this.criteria.getStatus()));
    }
    if (this.criteria.getTitle() != null) {
      predicatesList.add(builder.like(root.get("title"), "%" + this.criteria.getTitle() + "%"));
    }
    if (this.criteria.getFormData() != null) {
      predicatesList.add(builder.like(root.get("formData"), "%" + this.criteria.getFormData() + "%"));
    }
    if (this.criteria.getMetadata() != null) {
      predicatesList.add(builder.like(root.get("metadata"), "%" + this.criteria.getMetadata() + "%"));
    }
    if (this.criteria.getSubtype() != null && this.criteria.getSubtype().getResId() != null) {
      predicatesList.add(builder.equal(root.get("subtype").get("resId"), this.criteria.getSubtype().getResId()));
    }
    if (!StringTool.isNullOrEmpty(this.criteria.getDoi())) {
      predicatesList.add(builder.like(root.get("doi"), "%" + this.criteria.getDoi() + "%"));
    }
    if (this.criteria.getPmid() != null) {
      predicatesList.add(builder.like(root.get("pmid"), "%" + this.criteria.getPmid() + "%"));
    }
    if (!StringTool.isNullOrEmpty(this.criteria.getArchiveId())) {
      predicatesList.add(builder.equal(root.get("archiveId"), this.criteria.getArchiveId()));
    }
    if (!StringTool.isNullOrEmpty(this.fullName)) {
      List<String> listTerm = Arrays.stream(this.fullName.split(" ")).filter(term -> !StringTool.isNullOrEmpty(term))
              .collect(Collectors.toList());
      if (listTerm.size() > 0) {
        for (String term : listTerm) {
          Predicate likeFirstName = builder.like(root.get("creator").get("firstName"), "%" + term + "%");
          Predicate likeLastName = builder.like(root.get("creator").get("lastName"), "%" + term + "%");
          predicatesList.add(builder.or(likeFirstName, likeLastName));
        }
      }
    }

    if (this.lastUpdatedBefore != null) {
      predicatesList.add(builder.lessThanOrEqualTo(root.get("lastUpdate").get("when"), this.lastUpdatedBefore));
    }
  }

  public void setFullName(String fullName) {
    if (!StringTool.isNullOrEmpty(fullName)) {
      fullName = fullName.replaceAll(",", "");
    }
    this.fullName = fullName;
  }

  public void setLastUpdatedBefore(OffsetDateTime lastUpdatedBefore) {
    this.lastUpdatedBefore = lastUpdatedBefore;
  }
}
