package ch.unige.aou.business;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import ch.unige.solidify.auth.model.AuthApplicationRole;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.service.ResourceService;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.Sortable;
import ch.unige.aou.model.settings.ValueWithTranslations;

public abstract class AouResourceService<T extends Resource> extends ResourceService<T> {

  @Autowired
  protected SpecificationPermissionsFilter<T> specificationPermissionFilter;

  @Autowired
  protected AuthenticatedUserProvider authenticatedUserProvider;

  @Override
  public T findOne(String resId) {
    T item = super.findOne(resId);
    this.afterFind(item);
    return item;
  }

  public Optional<T> findById(String resId) {
    Optional<T> itemOpt = this.itemRepository.findById(resId);
    if (itemOpt.isPresent()) {
      T item = itemOpt.get();
      this.afterFind(item);
    }
    return itemOpt;
  }

  @Override
  public Page<T> findAll(Specification<T> spec, Pageable pageable) {
    Page<T> results = super.findAll(spec, pageable);
    if (results.hasContent()) {
      results.forEach(item -> this.afterFind(item));
    }
    return results;
  }

  @Override
  public Page<T> findAll(Pageable pageable) {
    Page<T> results = super.findAll(pageable);
    if (results.hasContent()) {
      results.forEach(item -> this.afterFind(item));
    }
    return results;
  }

  /**
   * Override if treatment must be done after the entity has been fetched from db
   *
   * @param item
   * @return
   */
  @Override
  protected T afterFind(T item) {
    return item;
  }

  protected ValueWithTranslations newValueWithEngFreTranslations(String value, String englishText, String frenchText) {
    ValueWithTranslations valueWithTranslations = new ValueWithTranslations(value);
    valueWithTranslations.addTranslation(AouConstants.LANG_CODE_ENGLISH, englishText);
    valueWithTranslations.addTranslation(AouConstants.LANG_CODE_FRENCH, frenchText);
    return valueWithTranslations;
  }

  public boolean authenticatedUserHasPriviledgedRole() {
    final String[] applicationRoles = new String[] { AuthApplicationRole.ADMIN_ID, AuthApplicationRole.ROOT_ID,
            AuthApplicationRole.TRUSTED_CLIENT_ID };

    final Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    if (authentication != null) {
      final java.util.Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();

      for (final String roleName : applicationRoles) {
        for (final GrantedAuthority authority : authorities) {
          if (authority.toString().equals(roleName)) {
            return true;
          }
        }
      }
    }
    return false;
  }

  public String cnIndividuToExternalUid(String cnIndividu) {
    return cnIndividu + "@unige.ch";
  }

  public String externalUidToCnIndividu(String externalUid) {
    return StringTool.substringBefore(externalUid, "@");
  }

  /**
   * Calculates a default sort value if the Sortable doesn't exist yet and has no sortValue property
   * Looks for values existing in database and sets a value placing the entity at the end of a sorted
   * list.
   *
   * @param item
   */
  public void setDefaultSortValue(Sortable item) {
    if (!this.itemRepository.existsById(item.getResId()) && item.getSortValue() == null) {
      Optional<Integer> maxValueOpt = this.getMaxSortValue(item);
      int newSortValue = maxValueOpt.isPresent() ? maxValueOpt.get() + AouConstants.ORDER_INCREMENT : AouConstants.ORDER_INCREMENT;
      item.setSortValue(newSortValue);
    }
  }

  // override in services managing Sortable entities
  public Optional<Integer> getMaxSortValue(Sortable item) {
    throw new SolidifyRuntimeException("method 'getMaxSortValue()' is not implemented");
  }
}
