package ch.unige.aou.business;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.publication.ContributorRole;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.publication.PublicationSubtypeContributorRole;
import ch.unige.aou.model.settings.Label;
import ch.unige.aou.model.settings.ValueWithTranslations;
import ch.unige.aou.repository.ContributorRoleRepository;
import ch.unige.aou.specification.ContributorRoleSpecification;

@Service
public class ContributorRoleService extends AouResourceService<ContributorRole> {
  private static final Logger log = LoggerFactory.getLogger(ContributorRoleService.class);

  private static final String ROLE_AUTHOR_EN = "Author";
  private static final String ROLE_COLLABORATOR_EN = "Collaborator";
  private static final String ROLE_DIRECTOR_EN = "Director";
  private static final String ROLE_EDITOR_EN = "Scientific publisher";
  private static final String ROLE_GUEST_EDITOR_EN = "Guest scientific publisher";
  private static final String ROLE_PHOTOGRAPHER_EN = "Photographer";
  private static final String ROLE_TRANSLATOR_EN = "Translator";
  private static final String ROLE_AUTHOR_FR = "Auteur";
  private static final String ROLE_COLLABORATOR_FR = "Collaborateur";
  private static final String ROLE_DIRECTOR_FR = "Directeur";
  private static final String ROLE_EDITOR_FR = "Editeur scientifique";
  private static final String ROLE_GUEST_EDITOR_FR = "Editeur scientifique invité";
  private static final String ROLE_PHOTOGRAPHER_FR = "Photographe";
  private static final String ROLE_TRANSLATOR_FR = "Traducteur";

  private ContributorRoleRepository contributorRoleRepository;
  private PublicationSubtypeService publicationSubtypeService;
  private LabeledLanguageService labeledLanguageService;

  public ContributorRoleService(ContributorRoleRepository contributorRoleRepository, PublicationSubtypeService publicationSubtypeService,
          LabeledLanguageService labeledLanguageService) {
    this.contributorRoleRepository = contributorRoleRepository;
    this.publicationSubtypeService = publicationSubtypeService;
    this.labeledLanguageService = labeledLanguageService;
  }

  @Override
  public ContributorRoleSpecification getSpecification(ContributorRole resource) {
    return new ContributorRoleSpecification(resource);
  }

  public ContributorRole findByValue(String name) {
    return this.contributorRoleRepository.findByValue(name);
  }

  /********************
   ** Initialization **
   ********************/

  @Transactional
  public void initDefaultData() {

    ValueWithTranslations valueAuthor = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_AUTHOR, ROLE_AUTHOR_EN,
            ROLE_AUTHOR_FR);
    //    ValueWithTranslations valueCollaborator = this.newValueWithEngFreTranslations("collaborator", ROLE_COLLABORATOR_EN, ROLE_COLLABORATOR_FR);
    ValueWithTranslations valueDirector = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_DIRECTOR, ROLE_DIRECTOR_EN,
            ROLE_DIRECTOR_FR);
    ValueWithTranslations valueEditor = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_EDITOR, ROLE_EDITOR_EN,
            ROLE_EDITOR_FR);
    ValueWithTranslations valueGuest = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_GUEST_EDITOR, ROLE_GUEST_EDITOR_EN,
            ROLE_GUEST_EDITOR_FR);
    ValueWithTranslations valuePhotographer = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_PHOTOGRAPHER,
            ROLE_PHOTOGRAPHER_EN, ROLE_PHOTOGRAPHER_FR);
    ValueWithTranslations valueTranslator = this.newValueWithEngFreTranslations(AouConstants.CONTRIBUTOR_ROLE_TRANSLATOR, ROLE_TRANSLATOR_EN,
            ROLE_TRANSLATOR_FR);

    /**
     * A1   Article scientifique
     * A2   Article professionnel
     * A3   Autre article
     */
    List<String> subtypeCodes = Arrays.asList("A1", "A2", "A3");
    // @formatter:off
    List<ValueWithTranslations> valuesWithTranslations = Arrays.asList(
            valueAuthor,
            valueTranslator,
            valuePhotographer
    );
    // @formatter:on
    this.createContributorRoles(subtypeCodes, valuesWithTranslations);

    /**
     * D1     Thèse de dictorat
     * D2     Master d'études avancées
     * D3     Master
     * D4     Thèse de privat-docent
     */
    subtypeCodes = Arrays.asList("D1", "D2", "D3", "D4");
    valuesWithTranslations = Arrays.asList(
            valueAuthor,
            valueDirector
    );
    this.createContributorRoles(subtypeCodes, valuesWithTranslations);

    /**
     * J1     Numéro de revue
     */
    subtypeCodes = Arrays.asList("J1");
    valuesWithTranslations = Arrays.asList(
            valueEditor,
            valueGuest
    );
    this.createContributorRoles(subtypeCodes, valuesWithTranslations);

    /**
     * L1     Livre
     * L3     Chapitre de livre
     * L5     Contribution à un dictionnaire / une encyclopédie
     * C1     Actes de conférence
     * C2     Présentation / Intervention
     * C3     Poster
     * C4     Chapitre d'actes
     * R1     Rapport de recherche
     * R2     Rapport technique
     * R3     Working paper
     * R4     Preprint
     */
    subtypeCodes = Arrays.asList("L1", "L3", "L5", "C1", "C2", "C3", "C4", "R1", "R2", "R3", "R4");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            valueAuthor,
            valueEditor,
            valueTranslator,
            valuePhotographer
    );
    // @formatter:on
    this.createContributorRoles(subtypeCodes, valuesWithTranslations);

    /**
     * L2     Ouvrage collectif
     */
    subtypeCodes = Arrays.asList("L2");
    valuesWithTranslations = Arrays.asList(
            valueEditor,
            valueTranslator,
            valuePhotographer
    );
    this.createContributorRoles(subtypeCodes, valuesWithTranslations);
  }

  private void createContributorRoles(List<String> subtypeCodes, List<ValueWithTranslations> valuesWithTranslations) {
    List<ContributorRole> contributorRoles = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      contributorRoles.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createContributorRolesForPublicationSubtypes(subtypeCodes, contributorRoles);
  }

  private ContributorRole createIfNotExists(ValueWithTranslations valueWithTranslations) {
    ContributorRole dft = this.findByValue(valueWithTranslations.getValue());
    if (dft == null) {
      dft = new ContributorRole();
      dft.setValue(valueWithTranslations.getValue());
      dft.getLabels().add(new Label(valueWithTranslations.getTranslation(AouConstants.LANG_CODE_ENGLISH),
              this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_ENGLISH)));
      dft.getLabels().add(new Label(valueWithTranslations.getTranslation(AouConstants.LANG_CODE_FRENCH),
              this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_FRENCH)));
      dft = this.save(dft);
      log.info("ContributorRole '{}' created", valueWithTranslations.getValue());
    }
    return dft;
  }

  private void createContributorRolesForPublicationSubtypes(List<String> subtypeCodes, List<ContributorRole> contributorRoles) {
    for (String subtypeCode : subtypeCodes) {
      this.createContributorRolesForPublicationSubtype(subtypeCode, contributorRoles);
    }
  }

  private void createContributorRolesForPublicationSubtype(String subtypeCode, List<ContributorRole> contributorRoles) {
    PublicationSubtype subtype = this.publicationSubtypeService.findByCode(subtypeCode);
    if (subtype != null) {
      int sortValue = AouConstants.ORDER_INCREMENT;
      boolean updated = false;
      for (ContributorRole dft : contributorRoles) {
        PublicationSubtypeContributorRole psDft = new PublicationSubtypeContributorRole();
        psDft.setPublicationSubtype(subtype);
        psDft.setContributorRole(dft);
        psDft.setSortValue(sortValue);

        if (!subtype.getPublicationSubtypeContributorRoles().contains(psDft)) {
          subtype.addPublicationSubtypeContributorRole(psDft);
          sortValue += AouConstants.ORDER_INCREMENT;
          updated = true;
        }
      }
      if (updated) {
        this.publicationSubtypeService.save(subtype);
        log.info("ContributorRoles configured for PublicationSubtype {}", subtype.getName());
      }
    } else {
      log.warn("Unable to configure ContributorRoles for PublicationSubtype {} that does not exist", subtypeCode);
    }
  }
}
