package ch.unige.aou.business;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.publication.DocumentFileType;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.publication.PublicationSubtypeDocumentFileType;
import ch.unige.aou.model.settings.Label;
import ch.unige.aou.model.settings.ValueWithTranslations;
import ch.unige.aou.repository.DocumentFileTypeRepository;
import ch.unige.aou.specification.DocumentFileTypeSpecification;

@Service
public class DocumentFileTypeService extends AouResourceService<DocumentFileType> {
  private static final Logger log = LoggerFactory.getLogger(DocumentFileTypeService.class);

  private static final String DOCUMENT_PUBLISHED_VERSION_EN = "Published version";
  private static final String DOCUMENT_ACCEPTED_VERSION_EN = "Accepted version";
  private static final String DOCUMENT_SUBMITTED_VERSION_EN = "Submitted version";
  private static final String DOCUMENT_PUBLISHED_VERSION_FR = "Version publiée";
  private static final String DOCUMENT_ACCEPTED_VERSION_FR = "Version acceptée";
  private static final String DOCUMENT_SUBMITTED_VERSION_FR = "Version soumise";

  private static final String RECORDING = "Recording";
  private static final String SUMMARY = "Summary";
  private static final String SUPPLEMENTAL_DATA = "Supplemental data";
  private static final String APPENDIX = "Appendix";
  private static final String EXTRACT = "Extract";
  private static final String TRANSLATION = "Translation";
  private static final String POSTER = "Poster";
  private static final String PREPRINT = "Preprint";
  private static final String ADDENDUM = "Addendum";
  private static final String CORRIGENDUM = "Corrigendum";
  private static final String ERRATUM = "Erratum";

  private DocumentFileTypeRepository documentFileTypeRepository;
  private PublicationSubtypeService publicationSubtypeService;
  private LabeledLanguageService labeledLanguageService;

  public DocumentFileTypeService(DocumentFileTypeRepository documentFileTypeRepository, PublicationSubtypeService publicationSubtypeService,
          LabeledLanguageService labeledLanguageService) {
    this.documentFileTypeRepository = documentFileTypeRepository;
    this.publicationSubtypeService = publicationSubtypeService;
    this.labeledLanguageService = labeledLanguageService;
  }

  @Override
  public DocumentFileTypeSpecification getSpecification(DocumentFileType resource) {
    return new DocumentFileTypeSpecification(resource);
  }

  public DocumentFileType findByValue(String name) {
    return this.documentFileTypeRepository.findByValue(name);
  }

  /********************
   ** Initialization **
   ********************/

  @Transactional
  public void initDefaultData() {

    /**
     * A1   Article scientifique
     * A2   Article professionnel
     * A3   Autre article
     */
    List<String> subtypeCodes = Arrays.asList("A1", "A2", "A3");
    // @formatter:off
    List<ValueWithTranslations> valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Article (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Article (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Article (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     *  C1    Actes de conférence
     */
    subtypeCodes = Arrays.asList("C1");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Proceedings (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Proceedings (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Proceedings (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     *  C2    Présentation / Intervention
     */
    subtypeCodes = Arrays.asList("C2");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Presentation",        "Presentation",    "Présentation"),
            this.newValueWithEngFreTranslations(RECORDING,           RECORDING,       "Enregistrement")
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(APPENDIX,             APPENDIX,             "Annexe"),
            this.newValueWithEngFreTranslations(SUPPLEMENTAL_DATA,    "Supplement",           "Supplément"),
            this.newValueWithEngFreTranslations(EXTRACT,              EXTRACT,              "Extrait"),
            this.newValueWithEngFreTranslations(SUMMARY,              SUMMARY,              "Résumé"),
            this.newValueWithEngFreTranslations(TRANSLATION,          TRANSLATION,          "Traduction")
    );
    // @formatter:on
    List<DocumentFileType> documentFileTypes = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      documentFileTypes.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createDocumentFileTypesForPublicationSubtypes(subtypeCodes, documentFileTypes, DocumentFileType.FileTypeLevel.SECONDARY);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * C3     Poster
     */
    subtypeCodes = Arrays.asList("C3");
    valuesWithTranslations = Arrays.asList(this.newValueWithEngFreTranslations(POSTER, POSTER, POSTER));
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * C4     Chapitre d'actes
     */
    subtypeCodes = Arrays.asList("C4");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Proceedings chapter (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Proceedings chapter (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Proceedings chapter (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * D1     Thèse
     */
    subtypeCodes = Arrays.asList("D1");
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Thesis", "Thesis", "Thèse"),
            this.newValueWithEngFreTranslations(DocumentFileType.IMPRIMATUR_VALUE, "Imprimatur (signed)", "Imprimatur (signé)"),
            this.newValueWithEngFreTranslations(DocumentFileType.PUBLICATION_MODE_VALUE, "Mode de publication (signed)",
                    "Mode de publication (signé)")
    );
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * D4     Thèse de privat-docent
     */
    subtypeCodes = Arrays.asList("D4");
    valuesWithTranslations = Arrays.asList(this.newValueWithEngFreTranslations("Thesis", "Thesis", "Thèse"));
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * D2     Master d'études avancées
     * D3     Master
     */
    subtypeCodes = Arrays.asList("D2", "D3");
    valuesWithTranslations = Arrays.asList(this.newValueWithEngFreTranslations("Master thesis", "Master thesis", "Mémoire de master"));
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * J1     Numéro de revue
     */
    subtypeCodes = Arrays.asList("J1");
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Issue (Published version)", "Issue (Published version)", "Numéro (Version publiée)")
    );
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * L1     Livre
     * L2     Ouvrage collectif
     */
    subtypeCodes = Arrays.asList("L1", "L2");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Book (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Book (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Book (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * L3     Chapitre de livre
     */
    subtypeCodes = Arrays.asList("L3");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Book chapter (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Book chapter (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Book chapter (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * L5     Contribution à un dictionnaire / une encyclopédie
     */
    subtypeCodes = Arrays.asList("L5");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations("Encyclopedia entry (Published version)", DOCUMENT_PUBLISHED_VERSION_EN, DOCUMENT_PUBLISHED_VERSION_FR),
            this.newValueWithEngFreTranslations("Encyclopedia entry (Accepted version)",  DOCUMENT_ACCEPTED_VERSION_EN,  DOCUMENT_ACCEPTED_VERSION_FR),
            this.newValueWithEngFreTranslations("Encyclopedia entry (Submitted version)", DOCUMENT_SUBMITTED_VERSION_EN, DOCUMENT_SUBMITTED_VERSION_FR)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * R1     Rapport de recherche
     * R2     Rapport technique
     */
    subtypeCodes = Arrays.asList("R1", "R2");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(DocumentFileType.REPORT, DocumentFileType.REPORT, "Rapport")
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * R3     Working paper
     */
    subtypeCodes = Arrays.asList("R3");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(DocumentFileType.WORKING_PAPER, DocumentFileType.WORKING_PAPER, DocumentFileType.WORKING_PAPER)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);

    /**
     * R4     Preprint
     */
    subtypeCodes = Arrays.asList("R4");
    // @formatter:off
    valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(PREPRINT, PREPRINT, PREPRINT)
    );
    // @formatter:on
    this.createPrincipalFileTypes(subtypeCodes, valuesWithTranslations);
    this.createSecondaryFileTypes(subtypeCodes);
    this.createCorrectiveFileTypes(subtypeCodes);
    this.createAdministrativeFileTypes(subtypeCodes);
  }

  private void createPrincipalFileTypes(List<String> subtypeCodes, List<ValueWithTranslations> valuesWithTranslations) {
    List<DocumentFileType> documentFileTypes = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      documentFileTypes.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createDocumentFileTypesForPublicationSubtypes(subtypeCodes, documentFileTypes, DocumentFileType.FileTypeLevel.PRINCIPAL);
  }

  private void createSecondaryFileTypes(List<String> subtypeCodes) {
    // @formatter:off
    List<ValueWithTranslations> valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(APPENDIX,             APPENDIX,             "Annexe"),
            this.newValueWithEngFreTranslations(SUPPLEMENTAL_DATA,    SUPPLEMENTAL_DATA,    "Données associées"),
            this.newValueWithEngFreTranslations(RECORDING,            RECORDING,            "Enregistrement"),
            this.newValueWithEngFreTranslations(EXTRACT,              EXTRACT,              "Extrait"),
            this.newValueWithEngFreTranslations(SUMMARY,              SUMMARY,              "Résumé"),
            this.newValueWithEngFreTranslations(TRANSLATION,          TRANSLATION,          "Traduction")
    );
    // @formatter:on

    List<DocumentFileType> documentFileTypes = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      documentFileTypes.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createDocumentFileTypesForPublicationSubtypes(subtypeCodes, documentFileTypes, DocumentFileType.FileTypeLevel.SECONDARY);
  }

  private void createCorrectiveFileTypes(List<String> subtypeCodes) {
    // @formatter:off
    List<ValueWithTranslations> valuesWithTranslations = Arrays.asList(
            this.newValueWithEngFreTranslations(ADDENDUM,             ADDENDUM,             ADDENDUM),
            this.newValueWithEngFreTranslations(CORRIGENDUM,          CORRIGENDUM,          CORRIGENDUM),
            this.newValueWithEngFreTranslations(ERRATUM,              ERRATUM,              ERRATUM),
            this.newValueWithEngFreTranslations("Retraction",           "Retraction",           "Rétraction")
    );
    // @formatter:on

    List<DocumentFileType> documentFileTypes = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      documentFileTypes.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createDocumentFileTypesForPublicationSubtypes(subtypeCodes, documentFileTypes, DocumentFileType.FileTypeLevel.CORRECTIVE);
  }

  private void createAdministrativeFileTypes(List<String> subtypeCodes) {

    // @formatter:off
    List<ValueWithTranslations> valuesWithTranslations = Arrays.asList(
      this.newValueWithEngFreTranslations(DocumentFileType.AGREEMENT_VALUE, "Agreement", "Contrat / Autorisation")
    );
    // @formatter:on

    List<DocumentFileType> documentFileTypes = new ArrayList<>();
    for (ValueWithTranslations valueWithTranslations : valuesWithTranslations) {
      documentFileTypes.add(this.createIfNotExists(valueWithTranslations));
    }
    this.createDocumentFileTypesForPublicationSubtypes(subtypeCodes, documentFileTypes, DocumentFileType.FileTypeLevel.ADMINISTRATION);
  }

  private DocumentFileType createIfNotExists(ValueWithTranslations valueWithTranslations) {
    DocumentFileType dft = this.findByValue(valueWithTranslations.getValue());
    if (dft == null) {
      dft = new DocumentFileType();
      dft.setValue(valueWithTranslations.getValue());
      dft.getLabels().add(new Label(valueWithTranslations.getTranslation(AouConstants.LANG_CODE_ENGLISH),
              this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_ENGLISH)));
      dft.getLabels().add(new Label(valueWithTranslations.getTranslation(AouConstants.LANG_CODE_FRENCH),
              this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_FRENCH)));
      dft = this.save(dft);
      log.info("DocumentFileType '{}' created", valueWithTranslations.getValue());
    }
    return dft;
  }

  private void createDocumentFileTypesForPublicationSubtypes(List<String> subtypeCodes, List<DocumentFileType> documentFileTypes,
          DocumentFileType.FileTypeLevel level) {
    for (String subtypeCode : subtypeCodes) {
      this.createDocumentFileTypesForPublicationSubtype(subtypeCode, documentFileTypes, level);
    }
  }

  private void createDocumentFileTypesForPublicationSubtype(String subtypeCode, List<DocumentFileType> documentFileTypes,
          DocumentFileType.FileTypeLevel level) {
    PublicationSubtype subtype = this.publicationSubtypeService.findByCode(subtypeCode);
    if (subtype != null) {
      int sortValue = AouConstants.ORDER_INCREMENT;
      boolean updated = false;
      for (DocumentFileType dft : documentFileTypes) {
        PublicationSubtypeDocumentFileType psDft = new PublicationSubtypeDocumentFileType();
        psDft.setPublicationSubtype(subtype);
        psDft.setDocumentFileType(dft);
        psDft.setLevel(level);
        psDft.setSortValue(sortValue);

        if (!subtype.getPublicationSubtypeDocumentFileTypes().contains(psDft)) {
          subtype.addPublicationSubtypeDocumentFileType(psDft);
          sortValue += AouConstants.ORDER_INCREMENT;
          updated = true;
        }
      }
      if (updated) {
        this.publicationSubtypeService.save(subtype);
        log.info("DocumentFileTypes configured for PublicationSubtype {}", subtype.getName());
      }
    } else {
      log.warn("Unable to configure DocumentFileTypes for PublicationSubtype {} that does not exist", subtypeCode);
    }
  }
}
