package ch.unige.aou.security.config;

import org.springframework.boot.actuate.autoconfigure.security.servlet.EndpointRequest;
import org.springframework.boot.actuate.info.InfoEndpoint;
import org.springframework.boot.actuate.metrics.export.prometheus.PrometheusScrapeEndpoint;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.web.filter.ForwardedHeaderFilter;

import ch.unige.solidify.auth.client.service.JwtOpaqueTokenIntrospector;
import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.unige.aou.AouConstants;
import ch.unige.aou.rest.UrlPath;

@Configuration
@EnableWebSecurity
@Profile("!sec-noauth")
public class ResourceServerConfig extends WebSecurityConfigurerAdapter {

  private final String[] privilegedRoles = { AuthApplicationRole.ADMIN_ID, AuthApplicationRole.TRUSTED_CLIENT_ID, AuthApplicationRole.ROOT_ID };
  private final String[] managementRoles = { AuthApplicationRole.TRUSTED_CLIENT_ID, AuthApplicationRole.ROOT_ID };

  private final JwtOpaqueTokenIntrospector jwtOpaqueTokenIntrospector;

  public ResourceServerConfig(JwtOpaqueTokenIntrospector jwtOpaqueTokenIntrospector) {
    this.jwtOpaqueTokenIntrospector = jwtOpaqueTokenIntrospector;
  }

  @Override
  public void configure(final HttpSecurity http) throws Exception {
    // @formatter:off
    http
      .csrf()
        .ignoringAntMatchers(AouConstants.getDisabledCsrfUrls())
      .and()
      .authorizeRequests()
      
        // Public URLs
        .antMatchers(AouConstants.getPublicUrls())
          .permitAll()
          
        // Public management endpoints
        .requestMatchers(EndpointRequest.to(InfoEndpoint.class, PrometheusScrapeEndpoint.class))
          .permitAll()

        // Management endpoints
        .requestMatchers(EndpointRequest.toAnyEndpoint())
          .hasAnyAuthority(this.managementRoles)
          
         // /admin/oauth2-clients
        .mvcMatchers(UrlPath.ADMIN_OAUTH2_CLIENTS + "/**")
          .hasAnyAuthority(this.privilegedRoles)

        .anyRequest().authenticated()
      .and()
      .sessionManagement()
        .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
        .and()
      .oauth2ResourceServer()
        .opaqueToken()
            .introspector(this.jwtOpaqueTokenIntrospector);
    // @formatter:on
  }

  /**
   * Register a ForwardedHeaderFilter for correct HATOEAS links behind a proxy See
   * https://stackoverflow.com/questions/30020188/how-to-configure-spring-hateoas-behind-proxy
   */
  @Bean
  FilterRegistrationBean<ForwardedHeaderFilter> forwardedHeaderFilter() {
    final FilterRegistrationBean<ForwardedHeaderFilter> bean = new FilterRegistrationBean<>();
    bean.setFilter(new ForwardedHeaderFilter());
    return bean;
  }

}
