package ch.unige.aou.service;

import java.util.Map;
import java.util.NoSuchElementException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationService;
import ch.unige.aou.controller.AouControllerAction;
import ch.unige.aou.model.publication.Publication;

@Service("publicationPermissionService")
public class PublicationPermissionService extends AbstractPermissionService {
  private static final Logger logger = LoggerFactory.getLogger(PublicationPermissionService.class);

  private PublicationService publicationService;

  public PublicationPermissionService(PersonService personService, PublicationService publicationService) {
    super(personService);
    this.publicationService = publicationService;
  }

  /******************************************************************************************************/

  public boolean isAllowedToCreate(Publication resource) {
    if (this.canBypassPermissionCheck()) {
      return true;
    }
    String personId = this.getPersonAuthenticatedId();
    return this.isAllowedToPerformActionOnResource(personId, resource, AouControllerAction.CREATE);
  }

  public boolean isAllowedToUpdate(String targetResId, Map<String, Object> updateMap) {
    if (this.canBypassPermissionCheck()) {
      return true;
    }
    String personId = this.getPersonAuthenticatedId();
    Publication existingResource = this.getExistingResource(targetResId);
    return this.isAllowedToPerformActionOnResource(personId, existingResource, AouControllerAction.UPDATE);
  }

  public boolean isAllowed(String targetResId, String actionString) {
    if (this.canBypassPermissionCheck()) {
      return true;
    }
    String personId = this.getPersonAuthenticatedId();
    Publication existingResource = this.getExistingResource(targetResId);
    if (existingResource == null) {
      throw new NoSuchElementException(targetResId);
    }
    AouControllerAction action = this.getControllerAction(actionString);
    return this.isAllowedToPerformActionOnResource(personId, existingResource, action);
  }

  public boolean isAllowed(String[] targetResIds, String actionString) {
    for (String id : targetResIds) {
      if (!this.isAllowed(id, actionString)) {
        return false;
      }
    }

    return true;
  }

  public boolean isAllowedToCreate() {
    return this.canBypassPermissionCheck() || this.hasUserApplicationRole();
  }

  /******************************************************************************************************/

  private Publication getExistingResource(String resId) {
    return this.publicationService.findOne(resId);
  }

  /******************************************************************************************************/

  private boolean isAllowedToPerformActionOnResource(String personId, Publication resourceToCheck, AouControllerAction action) {

    if (StringTool.isNullOrEmpty(personId)) {
      logger.error("unable to check permissions: personId is not set");
      return false;
    }

    final String publicationId = resourceToCheck.getResId();

    // @formatter:off
    return (this.hasUserApplicationRole()                          && this.isActionPermittedForUser(action)) ||
           (this.isResourceCreator(personId, publicationId)        && this.isActionPermittedForCreator(action)) ||
           (this.isResourceContributor(personId, publicationId)    && this.isActionPermittedForContributor(action)) ||
           (this.isAllowedToValidate(personId, publicationId)      && this.isActionPermittedForValidator(action));
    // @formatter:on
  }

  private boolean isActionPermittedForUser(AouControllerAction action) {
    // @formatter:off
    switch (action) {
      case CREATE:
      case GET:
      case CLONE:

      // manage comments
      case GET_DEPOSIT_COMMENT:
      case ADD_DEPOSIT_COMMENT:
      case LIST_DEPOSIT_COMMENTS:
      case UPDATE_DEPOSIT_COMMENT:
      case DELETE_DEPOSIT_COMMENT:

      // manage files
      case LIST_FILES:
      case DOWNLOAD_FILE:

      // validation
      case VALIDATE_METADATA:
        return true;
      default:
        return false;
    }
    // @formatter:off
  }

  private boolean isActionPermittedForCreator(AouControllerAction action) {
    return action == AouControllerAction.SUBMIT_FOR_VALIDATION || this.isActionPermittedForCreatorAndContributorAndValidator(action);
  }

  private boolean isActionPermittedForContributor(AouControllerAction action) {
    return action == AouControllerAction.SUBMIT_FOR_VALIDATION || this.isActionPermittedForCreatorAndContributorAndValidator(action);
  }

  private boolean isActionPermittedForValidator(AouControllerAction action) {
    switch (action) {
      case SUBMIT:
      case REJECT:
      case ASK_FEEDBACK:
      case LIST_VALIDATOR_COMMENTS:
      case RESUME:
      case SEND_BACK_FOR_VALIDATION:
      case ADD_DEPOSIT_VALIDATION_STRUCTURE:
      case DELETE_DEPOSIT_VALIDATION_STRUCTURE:
        return true;
      default:
        return this.isActionPermittedForCreatorAndContributorAndValidator(action);
    }
  }

  /**
   * Check if the action can be done by a person who is creator, contributor or validator of a deposit
   *
   * @param action
   * @return
   */
  private boolean isActionPermittedForCreatorAndContributorAndValidator(AouControllerAction action) {
    // @formatter:off
    switch (action) {
      // CRUD
      case GET:
      case HISTORY:
      case UPDATE:
      case DELETE:

      // manage files
      case GET_FILE:
      case ADD_DOCUMENT_FILE:
      case UPDATE_DOCUMENT_FILE:
      case DELETE_DOCUMENT_FILE:
      case UPLOAD_DOCUMENT_FILE:
        return true;

      default:
        return false;
    }
    // @formatter:on
  }

  /******************************************************************************************************/

  private boolean isAllowedToValidate(String personId, String publicationId) {
    Publication publication = this.publicationService.findOne(publicationId);

    return this.personService.isAllowedToValidate(personId, publication);
  }

  private boolean isResourceCreator(String personId, String publicationId) {
    return this.publicationService.isPersonPublicationCreator(personId, publicationId);
  }

  private boolean isResourceContributor(String personId, String publicationId) {
    return this.publicationService.isPersonPublicationContributor(personId, publicationId);
  }
}
